
/*
 * gsm1.c
 *
 *  Created on: 23 March 2021
 *      Author: Rajender
 */


#ifndef __gsm1_H
#define __gsm1_H

#define GSM_MAX_CHARS 256
uint8_t data_array1[GSM_MAX_CHARS];

unsigned int g_ui_OBD2UARTDataReceived2;
unsigned int g_ui_OBD2UARTDataReceived_count2;

#define pin_rx1 NRF_GPIO_PIN_MAP(1, 13)
#define pin_tx1 NRF_GPIO_PIN_MAP(1, 14)

#define SERIAL_BUFF_TX_SIZE 1
#define SERIAL_BUFF_RX_SIZE 1

#define SERIAL_FIFO_TX_SIZE 32
#define SERIAL_FIFO_RX_SIZE 32

void GSM_InterruptHandler(nrf_serial_t const *p_serial, nrf_serial_event_t event);

NRF_SERIAL_BUFFERS_DEF(serial1_buffs, SERIAL_BUFF_TX_SIZE, SERIAL_BUFF_RX_SIZE);
NRF_SERIAL_QUEUES_DEF(serial1_queues, SERIAL_FIFO_TX_SIZE, SERIAL_FIFO_RX_SIZE);

NRF_SERIAL_CONFIG_DEF(serial1_config, NRF_SERIAL_MODE_DMA,
                      &serial1_queues, &serial1_buffs, GSM_InterruptHandler, NULL);

NRF_SERIAL_UART_DEF(serial1_uarte, 1);

NRF_SERIAL_DRV_UART_CONFIG_DEF(m_uarte1_drv_config,
                      pin_rx1,pin_tx1,//ARDUINO_SDA_PIN, TX_PIN_NUMBER,
                      RTS_PIN_NUMBER, CTS_PIN_NUMBER,
                      NRF_UART_HWFC_DISABLED, NRF_UART_PARITY_EXCLUDED,
                      NRF_UART_BAUDRATE_115200,
                      UART_DEFAULT_CONFIG_IRQ_PRIORITY);


typedef struct
{
	unsigned char RxBuf[4096];
	unsigned int RxBuf_Len;
	unsigned int RxData_Receive;
	unsigned char TxBuf[4096];
	unsigned int TxBuf_Len;

}UartTelemetry_t;
UartTelemetry_t uartTelemetry;

typedef struct
{
	const char * cmd_name;
	unsigned int  cmd_Len;
}gsmcmd_t;

#define CMD_GSM_DATA(_x) _x, sizeof(_x)-1

const gsmcmd_t GSM_CMD_Table[]=
{
  {CMD_GSM_DATA("AT\r")},
  {CMD_GSM_DATA("ATE\r")},
  {CMD_GSM_DATA("ATE0\r")},
  {CMD_GSM_DATA("ATZ\r")},
  {CMD_GSM_DATA("ATE1\r")},
  {CMD_GSM_DATA("ATL1\r")},
  {CMD_GSM_DATA("AT+CPIN?\r")},
  {CMD_GSM_DATA("AT+CMGF=1\r")},
  {CMD_GSM_DATA("AT+CSCS=\"GSM\"\r")},
  {CMD_GSM_DATA("AT+CMGS=\"+917285907865\"\r\n")},

  /////////////////////// MQTT Commands  //////////////////////////////
  {CMD_GSM_DATA("AT+CMQTTSTART\r")},
  {CMD_GSM_DATA("AT+CMQTTACCQ=1,\"QWERTY\",0\r")},
  {CMD_GSM_DATA("AT+CMQTTWILLTOPIC=1,26\r")},
  {CMD_GSM_DATA("AT+CMQTTWILLMSG=1,4,0\r")},
  {CMD_GSM_DATA("Disc\r")},
  {CMD_GSM_DATA("AT+CMQTTCONNECT=1,\"tcp://io.adafruit.com: 1883\",120,1,\"himanc2\",\"aio_xFOr446u5ZPiSP9GbuLLu3Ee4LC3\"\r")},

  //****************  For Publish to a Topic ********************//
  {CMD_GSM_DATA("AT+CMQTTTOPIC=1,26\r")},
  {CMD_GSM_DATA("himanc2/feeds/welcome-feed\r")},
  {CMD_GSM_DATA("AT+CMQTTPAYLOAD=1,2\r")},
  {CMD_GSM_DATA("44\r")},


  //****************  For Subscribe a Topic ********************//
  {CMD_GSM_DATA("AT+CMQTTSUBTOPIC=1,26,0\r")},
  {CMD_GSM_DATA("himanc2/feeds/welcome-feed\r")},
  {CMD_GSM_DATA("AT+CMQTTSUB=1\r")},
  {CMD_GSM_DATA("AT\r")},
  {CMD_GSM_DATA("AT\r")},
  {CMD_GSM_DATA("AT\r")},
  {CMD_GSM_DATA("AT\r")}

};

/********************* GSM Commands ******************************/
#define AT                      0
#define ATE                     1
#define ATE0 			2
#define ATZ 			3
#define ATE1                    4
#define ATL1                    5
#define AT_CPIN                 6
#define AT_CMGF                 7
#define AT_CSCS                 8
#define AT_CMGS                 9

#define AT_CMQTTSTART           10
#define AT_CMQTTACCQ            11
#define AT_CMQTTWILLTOPIC       12
#define AT_CMQTTWILLMSG         13
#define AT_CMQTTCONNECT         14

#define AT_CMQTTTOPIC           15
#define AT_CMQTTPAYLOAD         16

#define AT_CMQTTSUBTOPIC        17
#define AT_CMQTTSUB             18
/************************************************************************/

#define GSM_RESULT_BASE_NUM      (0)       ///< Global error base


// ERROR CODES
#define GSM_OPERATION_SUCCEEDED           (GSM_RESULT_BASE_NUM + 0) //
#define GSM_FAILED                        (GSM_RESULT_BASE_NUM + 1) //
#define GSM_BAD_UTF8_STRING               (GSM_RESULT_BASE_NUM + 2) //
#define GSM_SOCK_CONN_FAIL                (GSM_RESULT_BASE_NUM + 3) //
#define GSM_SOCK_CREATE_FAIL              (GSM_RESULT_BASE_NUM + 4) //
#define GSM_SOCK_CLOSE_FAIL               (GSM_RESULT_BASE_NUM + 5) //
#define GSM_MSG_RCV_FAIL                  (GSM_RESULT_BASE_NUM + 6) //
#define GSM_NETWORK_OPEN_FAIL             (GSM_RESULT_BASE_NUM + 7) //
#define GSM_NETWORK_CLOSE_FAIL            (GSM_RESULT_BASE_NUM + 8) //
#define GSM_NETWORK_NOT_OPENED            (GSM_RESULT_BASE_NUM + 9) //
#define GSM_CLIENT_INDEX_ERROR            (GSM_RESULT_BASE_NUM + 10) //
#define GSM_NO_CONNECTION                 (GSM_RESULT_BASE_NUM + 11) //
#define GSM_INVALID_PARAMETER             (GSM_RESULT_BASE_NUM + 12) //
#define GSM_NOT_SUPPORTED_OPERATION       (GSM_RESULT_BASE_NUM + 13) //
#define GSM_CLIENT_BUSY                   (GSM_RESULT_BASE_NUM + 14) //
#define GSM_REQUIRE_CONN_FAIL             (GSM_RESULT_BASE_NUM + 15) //
#define GSM_SOCK_SENDING_FAIL             (GSM_RESULT_BASE_NUM + 16) //
#define GSM_TIMEOUT                       (GSM_RESULT_BASE_NUM + 17) //
#define GSM_TOPIC_EMPTY                   (GSM_RESULT_BASE_NUM + 18) //
#define GSM_CLIENT_USED                   (GSM_RESULT_BASE_NUM + 19) //
#define GSM_CLIENT_NOT_AQUIRED            (GSM_RESULT_BASE_NUM + 20) //
#define GSM_CLIENT_NOT_RELEASED           (GSM_RESULT_BASE_NUM + 21) //
#define GSM_LENGTH_OUT_OF_RANGE           (GSM_RESULT_BASE_NUM + 22) //
#define GSM_NETWORK_OPENED                (GSM_RESULT_BASE_NUM + 23) //
#define GSM_PACKET_FAIL                   (GSM_RESULT_BASE_NUM + 24) //
#define GSM_DNS_ERROR                     (GSM_RESULT_BASE_NUM + 25) //
#define GSM_SOCKET_CLOSED_BY_SERVER       (GSM_RESULT_BASE_NUM + 26) //
#define GSM_CRF_UNAUTH_PROTOCOL_VERSION   (GSM_RESULT_BASE_NUM + 27) //
#define GSM_CRF_IDENTIFIER_REJECTED       (GSM_RESULT_BASE_NUM + 28) //
#define GSM_CRF_SERVER_UNAVAILABLE        (GSM_RESULT_BASE_NUM + 29) //
#define GSM_CRF_BAD_USER_NAME_OR_PASSWORD (GSM_RESULT_BASE_NUM + 30) //
#define GSM_CRF_NOT_AUTHORIZED            (GSM_RESULT_BASE_NUM + 31) //
#define GSM_HANDSHAKE_FAIL                (GSM_RESULT_BASE_NUM + 32) //
#define GSM_NOT_SET_CERTIFICATE           (GSM_RESULT_BASE_NUM + 33) //
#define GSM_OPEN_SSL_SESSION_FAIL         (GSM_RESULT_BASE_NUM + 34) //


/********************* Error Codes of  GSM *********************/
#define GSM_SUCCESS                             0
#define GSM_DATA_VALIDATION_SUCCESS             1
#define GSM_RECIVED_LESS_CHARS			2
#define GSM_RECIVED_MORE_CHARS			3
#define GSM_RECV_UNKNOWN_DATA			4
#define GSM_RECV_FULL_DATA			5
#define GSM_RECV_STX				6
#define GSM_SER_TIMEOUT				7
#define GSM_DATA_PARSE_FAIL			8
#define GSM_DATA_VALIDATION_ERROR		9

 
unsigned char flushuartbuffer(void)
{
	unsigned char data;
	//HAL_StatusTypeDef uartstatus;
	unsigned char count =0;
	while(nrf_serial_read(&serial1_uarte, &data, sizeof(data), NULL, 1)==NRF_SUCCESS)
	{
		count++;
		if(count>128){break;}
	};
	return 0;
}

unsigned short GSMUART_interrupt_enable(unsigned char * o_puc_RecData, unsigned int size,unsigned int rec_count)
{
        NVIC_EnableIRQ(UARTE1_IRQn);
	return (nrf_serial_read(&serial1_uarte, (uint8_t *)&o_puc_RecData[0],size,NULL,rec_count));
}

void GSM2UART_interrupt_disable(void)
{
	NVIC_DisableIRQ(UARTE1_IRQn);
}

unsigned char validate_cmd(unsigned int i_ui_cmd,char* buf,unsigned int respbuflen)
{
	unsigned char status=GSM_DATA_VALIDATION_SUCCESS;
	char *match;
	char *p ;
	unsigned char pidmap[4 * 4];
	//char temparray[32];
	//int pid=0xff;

	switch(i_ui_cmd)
	{
	case AT_CMQTTSUB:
          if((memcmp(buf,"AT\rOK\r>",strlen("AT\rOK\r>"))!=0)){return status=GSM_DATA_VALIDATION_ERROR;}
          break;
        }
}


unsigned short GSM_SendData(unsigned char * i_data, unsigned int i_data_Len)		//putting the data on SBUF
{
	return (nrf_serial_write(&serial1_uarte, i_data, i_data_Len, NULL, 1000));
}

unsigned short GSM_ReceiveOneByte(unsigned char c)		//putting the data on SBUF
{
	return (nrf_serial_read(&serial1_uarte, &c, 1,NULL, 2));
}

void GSM_InterruptHandler(nrf_serial_t const *p_serial, nrf_serial_event_t event)
{
	unsigned int l_char_count;
	l_char_count=1;
  uint32_t err_code;
  char bb;
  static uint8_t index1=0;
  while(1){
  switch (event)
  {
    case NRF_SERIAL_EVENT_RX_DATA:
      nrf_serial_read(&serial1_uarte, &bb, sizeof(bb), NULL, 2000);
      data_array1[index1++] = bb;
     
      break;
    case NRF_SERIAL_EVENT_TX_DONE:
      //NRF_LOG_INFO("Sent-1");
      break;
    case NRF_SERIAL_EVENT_DRV_ERR:
      APP_ERROR_CHECK(nrf_serial_uninit(&serial1_uarte));
      ret_code_t ret;
      ret = nrf_serial_init(&serial1_uarte, &m_uarte1_drv_config, &serial1_config);
      APP_ERROR_CHECK(ret);
      break;
    case NRF_SERIAL_EVENT_FIFO_ERR:
      break;
    default:
      break;
  }
  }
  	g_ui_OBD2UARTDataReceived_count2=g_ui_OBD2UARTDataReceived_count2+l_char_count;
	g_ui_OBD2UARTDataReceived2=1;

}
unsigned char sendCommand_resp(unsigned int i_ui_cmd, char* buf, unsigned int bufsize, unsigned int *respbuflen, int timeout)
{
  unsigned char status=1;
  unsigned char Tempbuff[GSM_MAX_CHARS]={0x00};
  *respbuflen=0;
  memset(&uartTelemetry, 0x00, sizeof(UartTelemetry_t));
  memset(Tempbuff,0x00,sizeof(Tempbuff));

  g_ui_OBD2UARTDataReceived2=0;
  g_ui_OBD2UARTDataReceived_count2=0;
  memcpy(uartTelemetry.TxBuf,GSM_CMD_Table[i_ui_cmd].cmd_name,GSM_CMD_Table[i_ui_cmd].cmd_Len);
  uartTelemetry.TxBuf_Len=GSM_CMD_Table[i_ui_cmd].cmd_Len;
  flushuartbuffer();
  GSMUART_interrupt_enable(uartTelemetry.RxBuf,1,6);
  GSM_SendData(uartTelemetry.TxBuf,uartTelemetry.TxBuf_Len);

  while(timeout--)
  {
    if(g_ui_OBD2UARTDataReceived2!=0)
    {
      GSM2UART_interrupt_disable();
      g_ui_OBD2UARTDataReceived2=0;

      memcpy(Tempbuff+strlen((char *)Tempbuff),uartTelemetry.RxBuf,strlen((char*) uartTelemetry.RxBuf));
      memset(uartTelemetry.RxBuf,0x00,strlen((char *)uartTelemetry.RxBuf));


      //if(strchr((const char*)Tempbuff,'>') !=NULL)
      {
              status=0;
              break;                                                                                 
      }

    }
    nrf_delay_ms(1);
  }
  GSM2UART_interrupt_disable();
  //	if(i_ui_cmd == ELM327_PID_MAP40){
//		if(strchr((const char*)Tempbuff,'>') !=NULL){
//
//			    strcpy(str,buf);
//			    str1[0]=0x1a;
//                str1[2]="\r";
//			    strcat(str,str1);
//			HAL_UART_Transmit(&huart2,(uint8_t *)str, strlen(str), 1000);
//		}
//	}

	

//	if((status!=0)){
//		memset(uartTelemetry.RxBuf, 0x00, sizeof(uartTelemetry.RxBuf));
//		return OBD2UART_SER_TIMEOUT;
//	}
//	if((i_ui_cmd==ELM327_ATZ) || (i_ui_cmd==ELM327_ATE0))
//	{
//		*respbuflen=strlen((char*)Tempbuff);
//		memcpy((unsigned char*)(buf),Tempbuff,*respbuflen);
//		g_ui_OBD2UARTDataReceived_count2 =0;
//
//	}
//	else
//	{
		*respbuflen=strlen((char*)Tempbuff);
		memcpy((unsigned char*)(buf),Tempbuff,*respbuflen);
		g_ui_OBD2UARTDataReceived_count2 =0;
//	}
	return GSM_SUCCESS;
}


#endif